use actix_web::{web, HttpResponse, Responder};
use chrono::Days;
use sqlx::PgPool;

use crate::{
    endpoints::{availability::NewOrEditAvailabilityTemplate, IdPath},
    models::{find_free_date_time_slots, Availability, User},
    utils::{ApplicationError, TemplateResponse},
};

#[actix_web::get("/availabillity/edit/{id}")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    let Some(availability) = Availability::read_by_id(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    if availability.user_id != user.id {
        return Err(ApplicationError::Unauthorized);
    }

    let availabilities =
        Availability::read_by_user_and_date(pool.get_ref(), user.id, &availability.start.date())
            .await?;

    let slot_suggestions = find_free_date_time_slots(&availabilities)
        .into_iter()
        .filter(|(a, b)| *b == availability.start || *a == availability.end)
        .collect();

    let template = NewOrEditAvailabilityTemplate {
        user: user.into_inner(),
        date: availability.start.date(),
        enddate: Some(availability.end.date()),
        id: Some(path.id),
        start: Some(availability.start.time()),
        end: Some(availability.end.time()),
        comment: availability.comment.as_deref(),
        slot_suggestions,
        datetomorrow: availability
            .start
            .date()
            .checked_add_days(Days::new(1))
            .unwrap(),
    };

    Ok(template.to_response()?)
}
